% M = com_new (d, m)
% Create a continuous memory
% - that consists of m units
% - to store d-dimensional vectors
function M = com_new(d, m, X, params)

% Add already some vectors if provided
if ~exist('X'), return; end
if ~exist ('params')
    params.aggregator = 'sum';
    params.allocator = 'regular2';
end

M.stored = struct;
M.com = zeros (d, m, 'single');       % The aggregating vectors

for i = 1:m
    M.stored(i).ids = [];
    M.stored(i).vec = zeros(d, 0, 'single');  % store the individual vectors
end

assert (size(X, 1) == d);
n = size (X, 2);


% Allocation of the different vector to different memories
switch params.allocator
    case 'regular'
        perm_inds = randperm(n);
        for i = 1:m
            istart = 1 + floor (((i-1) * n)/ m);
            iend = floor ((i * n)/ m);
            M.stored(i).ids = perm_inds(istart:iend);
        end
        
    case 'regular2'
        for i = 1:m
            M.stored(i).ids = find(mod(1:n,m) == i-1);
        end
        
    case 'kmeans'
        [~,I] = yael_kmeans (X, m,'seed', 1);
        for i = 1:m
            M.stored(i).ids = find (I' == i);
        end
    case 'comu_kmeans'
        [C,I] = sphere_kmeans (X, m,params.aggregator);
        for i = 1:m
            M.stored(i).ids = find (I' == i);
        end
    otherwise
        error('Invalid selection')
end


switch params.allocator
    case 'comu_kmeans'
        
        M.com = C;
    otherwise
        % Compute the representative vector for each memory unit if they are not
        % alread computed
        fprintf('Computing the representative for each memory unit\n');
        for i = 1:m
            
            ids = M.stored(i).ids;
            
            nids = numel (ids);
            M.stored(i).vec = X(:,ids);
            
            switch params.aggregator
                case 'sum'
                    M.com(:, i) = sum (X(:,ids), 2);
                    
                case 'pinv'
                    % Solve the system directly with pseudo-inverse
                    s = ones (nids, 1);
                    M.com(:, i) = pinv(X(:,ids)') * s;
                                        
                case 'gs'
                    G = zeros (d, nids, 'single');
                    G(:,1) = X(:,ids(1));
                    M.com (:, i) = G(:,1);
                    for j=2:nids
                        G(:,j) = X(:, ids(j));
                        
                        % Disregard zero vectors
                        if(sum(G(:,j))~=0)
                            
                            % orthogonalize the vector: remove the contribution of previous ones
                            pGX = G(:,1:j-1)' * X(:, ids(j));
                            alpha = pGX' ./ sum(G(:,1:j-1).^2);
                            alpha(find(isnan(alpha))) = 0;
                            G(:,j) = G(:,j) - sum (bsxfun (@times, G(:,1:j-1), alpha), 2);
                            
                            % Compute the factor that ensures to get similarity 1
                            G(:,j) = G(:,j) * (1 - sum (pGX)) / (G(:, j)' * X(:, ids(j)));
                            
                        end
                    end
                    M.com (:, i) = sum (G, 2);
                    
            end
            
        end
end
