function [C, idx] = sphere_kmeans (X, k,mode)

assert(k>0,'k must be above 0');

if nargin < 3
    mode = 'sum';
end



niter = 10;
slice = 256;

n = size (X, 2);
d = size (X, 1);

fprintf('Running spherical k-means, n= %d\n',n);

assert(k<n,'k must be less than n');

% Initialization with random vectors
C = X(:, randperm(n, k));

% Normalization function
% vnormalize = inline ('bsxfun (@times, x, (sum (x.^2)) .^ (-0.5))', 'x');

switch mode
    case 'sum'
        updatefunc = @(X,g) sum(X(:,g), 2)';
    case 'pinv'
        updatefunc = @(X,g) pinv(X(:,g)')*ones(numel(g),1);
        
end

% Only one cluster case
if k==1
    C = updatefunc(X,1:n);
    idx = ones(n,1);
    return
end

for i = 1:niter
%     fprintf('iter=%d\n',i);
    
    % Assign step: per slice to limit memory overhead
    m1 = -inf * ones(1,n);
    idx = zeros (1, n);
    
    for q = 1:slice:k;
        idC = q:min(q+slice-1, k);
        [m2, id2] = max (score_normalize(C(:,idC),X));
        
        id2 = idC (id2);
        T = m1 < m2;
        idx = (1-T) .* idx + T .* id2;
        m1  = max (m1, m2);
    end
    
    
    % Identify how to group these vectors into clusters
    G = accumarray(idx',1:n,[k 1],@(x){x});
    
    % Reassign empty clusters
    C(:,cellfun('isempty',G)) = X(:, randperm(n, sum(cellfun('isempty',G))));
    
    % Update stage
    for j=find(~cellfun('isempty',G))'
        C(:,j) = updatefunc(X,G{j});
    end
    
    
end

idx=idx';